# Plugins

Plugins are extensions that are used to process online the data flowing through the connections of the model during the simulation. `Jusdl` provides some simple `Plugin`s such as [`Plugins.Lyapunov`](@ref), [`Plugins.Fft`](@ref), [`Plugins.Mean`](@ref), [`Plugins.Std`](@ref), [`Plugins.Variance`](@ref). These tools are specialized tools that are used for specialized data processing. In addition to the plugins that are provided by `Jusdl`, it is also possible to write new plugins that focus on different specialized data processing. The fundamental importance of `Plugin`s is that they the **online simulation data processing** possible. 

The `Plugin`s are mostly used with [Sinks](@ref). In `Jusdl`, the `Sink`s are used to *sink* simulation data flowing through the connections of the model. When a `Sink` is equipped with a proper `Plugin` according to the data processing desired, then the data flowing into the `Sink` is processed. For example, consider that a `Writer` is equipped with a `Lyapunov` plugin. During the simulation, data flowing into the `Writer` is processed to compute the maximum Lyapnunov exponent, and this computed maximum Lyapunov exponents are recorded in the file of the `Writer`. Similarly, if a `Printer` is equipped with an `Fft` plugin, then Fast Fourier transform of the data flowing into the `Printer` is printed on the console.

## Data processing via Plugins 
Each `Plugin` must have a `process` function which does the data processing. The first argument of the `process` function is the `Plugin` and the second argument is the data to be processed. Here are some of the methods of `process` function

```@docs 
process
```

## Defining New Plugins
New plugins can be defined in `Jusld` and having they are defined properly they can work just expected. To define a new plugin, we must first define the plugin type 
```@repl plugin_ex 
using Jusdl # hide 
import Jusdl.Plugins.AbstractPlugin
struct NewPlugin <: AbstractPlugin
    # Parameters of NewPlugin
end
```

!!! warning
    Note that to the `NewPlugin` is defined to be a subtype of `AbstractPlugin`. This is important for the `NewPlugin` to work as expected.

Since each plugin must have `process` method, `Jusdl.Plugins.process` function must be imported and dispatched.

```@repl plugin_ex
import Jusdl.Plugins.process
function process(plg::NewTemplate, x)
    # Define the process according to plg
end
```
At this point, `NewPlugin` is ready to be used. 

## Full API
```@docs 
Plugins.Lyapunov
Plugins.Fft
Plugins.Mean
Plugins.Std
Plugins.Variance
```

